@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: base64_lua.c,v 1.4 2005/06/03 16:29:19 alessiofender Exp $
+ * $Id: base64_lua.c,v 1.5 2005/09/26 22:22:44 gareuselesinge Exp $
  * This file is part of FreePOPs (http://www.freepops.org)                    *
  * This file is distributed under the terms of GNU GPL license.               *
  ******************************************************************************/
@@ -19,7 +19,6 @@
 #include "lauxlib.h"
 #include "base64.h"
 
-/* function: dothack */
 static int lua_base64_encode(lua_State* L)
 {
 const char * s = luaL_checkstring(L,1);
@@ -32,8 +31,21 @@ if (tmp != s)
 return 1;
 }
 
+static int lua_base64_decode(lua_State* L)
+{
+const char * s = luaL_checkstring(L,1);
+
+char * tmp = base64dec(s,lua_strlen(L,1));
+lua_pushstring(L,tmp);
+if (tmp != s)
+	free(tmp);
+
+return 1;
+}
+
 static const struct luaL_reg stringhack_f [] = {
   {"encode",lua_base64_encode},
+  {"decode",lua_base64_decode},
   {NULL,NULL}
 };
 
