@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: altsocklib.c,v 1.9 2005/06/11 00:43:59 gareuselesinge Exp $
+ * $Id: altsocklib.c,v 1.11 2005/11/19 15:39:34 ncocchiaro Exp $
  * This file is part of liberopops (http://liberopops.sf.net)                 *
  * This file is distributed under the terms of GNU GPL license.               *
  ******************************************************************************/
@@ -50,7 +50,7 @@
  #include <errno.h>
 
  /* on some libc there is no _r support */
- #ifndef strerror_r
+ #if !defined(strerror_r) || defined(OSXSTC)
 
   # include <pthread.h>
   pthread_mutex_t strerror_lock = PTHREAD_MUTEX_INITIALIZER;
@@ -318,7 +318,7 @@ return sd;
 int sockinfo(int sd, char *info)
 {
     struct sockaddr_in sin;
-#if !defined(WIN32) && !defined(BEOS)
+#if !defined(WIN32) && !defined(BEOS) && !defined(OSXSTC)
     socklen_t len = sizeof(sin);
 #else
     int len = sizeof(sin);
