@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: regularexp.c,v 1.3 2004/10/15 19:20:14 gareuselesinge Exp $
+ * $Id: regularexp.c,v 1.4 2005/09/25 11:55:53 gareuselesinge Exp $
  * This file is part of liberopops (http://liberopops.sf.net)                 *
  * This file is distributed under the terms of GNU GPL license.               *
  ******************************************************************************/
@@ -25,7 +25,13 @@ regmatch_t regfind(const char* from,const char* exp)
 {
 int errcode;
 regex_t r;
+#ifndef __sun
 regmatch_t pm[1]={{-1,-1}};
+#else
+regmatch_t pm[1];
+pm[0].rm_so = -1;
+pm[0].rm_eo = -1;
+#endif
 
 if(from == NULL)
 	return pm[0];
