@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: regularexp_lua.c,v 1.5 2005/06/03 16:29:20 alessiofender Exp $
+ * $Id: regularexp_lua.c,v 1.6 2005/09/25 11:55:53 gareuselesinge Exp $
  * This file is part of FreePOPs (http://www.freepops.org)                    *
  * This file is distributed under the terms of GNU GPL license.               *
  ******************************************************************************/
@@ -68,7 +68,13 @@ int nmatch = 0;
 int max;
 const char *text;
 regex_t *pr;
+#ifndef __sun
 regmatch_t pm[1]={{-1,-1}};
+#else
+regmatch_t pm[1];
+pm[0].rm_so = -1;
+pm[0].rm_eo = -1;
+#endif
 
 pr = check_regularexp(L);
 text = luaL_checkstring(L,2);
@@ -100,7 +106,13 @@ int max;
 size_t cur=0;
 const char *text;
 regex_t *pr;
+#ifndef __sun
 regmatch_t pm[1]={{-1,-1}};
+#else
+regmatch_t pm[1];
+pm[0].rm_so = -1;
+pm[0].rm_eo = -1;
+#endif
 
 pr = check_regularexp(L);
 text = luaL_checkstring(L,2);
