@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: winsystray.c,v 1.6 2004/09/30 18:25:38 gareuselesinge Exp $
+ * $Id: winsystray.c,v 1.8 2005/10/26 09:13:07 gareuselesinge Exp $
  * This file is part of liberopops (http://liberopops.sf.net)                 *
  * This file is distributed under the terms of GNU GPL license.               *
  ******************************************************************************/
@@ -272,7 +272,8 @@ do
 				}
 			else
 				{
-				ERROR_ABORT("Wrong arg string\n");
+				ERROR_PRINT("Wrong arg string \n");
+				goto abort;
 				}
 			
 			}
@@ -320,7 +321,8 @@ do
 				}
 			else
 				{
-				ERROR_ABORT("Wrong arg string\n");
+				ERROR_PRINT("Wrong arg string \n");
+				goto abort;
 				}
 			}
 		}
@@ -329,6 +331,7 @@ while(p.begin != -1);
 
 *argv=argv_win32;
 
+abort:
 free(str);
 return n;
 }
