@@ -7,17 +7,20 @@
 
 -- Globals
 --
-PLUGIN_VERSION = "0.1.0"
+PLUGIN_VERSION = "0.1.3a"
 PLUGIN_NAME = "hotmail.com"
 PLUGIN_REQUIRE_VERSION = "0.0.25"
 PLUGIN_LICENSE = "GNU/GPL"
 PLUGIN_URL = "http://www.freepops.org/download.php?file=hotmail.lua"
 PLUGIN_HOMEPAGE = "http://www.freepops.org/"
-PLUGIN_AUTHORS_NAMES = {"Russell Schwager"}
-PLUGIN_AUTHORS_CONTACTS = {"russells (at) despammed (.) com"}
-PLUGIN_DOMAINS = {"@hotmail.com","@msn.com","@webtv.com","@charter.com",
-	"@compaq.net","@passport.com", "@hotmail.de", "@hotmail.it", "@hotmail.co.uk", 
-        "@hotmail.co.jp", "@hotmail.fr", "@messengeruser.com"}
+PLUGIN_AUTHORS_NAMES = {"Russell Schwager", "D. Milne" }
+PLUGIN_AUTHORS_CONTACTS = {"russells (at) despammed (.) com", "drmilne (at) safe-mail (.) net"}
+PLUGIN_DOMAINS = { "@hotmail.com","@msn.com","@webtv.com",
+      "@charter.com", "@compaq.net","@passport.com", 
+      "@hotmail.de", "@hotmail.it", "@hotmail.co.uk", 
+      "@hotmail.co.jp", "@hotmail.fr", "@messengeruser.com",
+      "@hotmail.com.ar", "@hotmail.co.th", "@hotmail.com.tr"  
+      }
 PLUGIN_PARAMETERS = {
 	{name="folder", description={
 		it=[[La cartella che vuoi ispezionare. Quella di default &egrave; Inbox, gli altri valori possibili sono: Junk, Trash, Draft, Sent.]],
@@ -30,7 +33,12 @@ Parameter is used to force the plugin to empty the trash when it is done
 pulling messages.  Set the value to 1.]]
 		}	
 	},
-
+	{name = "markunread", description = {
+		it = [[ Viene usato per far s&igrave; che il plugin segni come non letti i messaggi che scarica. Se il valore &egrave; 1 questo comportamento viene attivato.]],
+		en = [[ Parameter is used to have the plugin mark all messages that it
+pulls as unread.  If the value is 1, the behavior is turned on.]]
+		}
+	},
 }
 PLUGIN_DESCRIPTIONS = {
 	it=[[
@@ -83,7 +91,8 @@ local globals = {
   strLoginPostUrlPattern3='g_DO."%s".="([^"]+)"',
   strLoginPostUrlPattern4='var g_QS="([^"]+)";',
   strLoginPostUrlPattern5='name="PPFT" id="[^"]+" value="([^"]+)"',
-  strLoginDoneReloadToHMHome='URL=([^"]+)"',
+  strLoginDoneReloadToHMHome1='URL=([^"]+)"',
+  strLoginDoneReloadToHMHome2='top%.location%.replace%("([^"]+)"',
 
   -- Get the crumb value that is needed for every command
   --
@@ -93,6 +102,10 @@ local globals = {
   --
   strImgServerPattern = 'img src="(http://[^/]*)/spacer.gif"',
 
+  -- Folder id pattern
+  --
+  strFolderPattern = '<a href="[^"]+curmbox=([^&]+)&[^"]+" >', 
+
   -- Pattern to determine if we have no messages
   --
   strMsgListNoMsgPat = "(<td colspan=10>)", --"(There are no messages in this folder)",
@@ -147,6 +160,7 @@ local globals = {
   strCmdMsgViewRaw = "&raw=0",
   strCmdEmptyTrash = "http://%s/cgi-bin/dofolders?_HMaction=DoEmpty&curmbox=F000000004&a=%s&i=F000000004",
   strCmdLogout = "http://%s/cgi-bin/logout",
+  strCmdFolders = "http://%s/cgi-bin/folders?&curmbox=F000000001&a=%s",
 }
 
 -- ************************************************************************** --
@@ -164,10 +178,44 @@ internalState = {
   strDomain = nil,
   strCrumb = nil,
   strMBox = nil,
+  strMBoxName = nil,
   bEmptyTrash = false,
   loginTime = nil,
+  bMarkMsgAsUnread = false,
 }
 
+-- ************************************************************************** --
+--  Debugging functions
+-- ************************************************************************** --
+
+-- Set to true to enable Raw Logging
+--
+local ENABLE_LOGRAW = false
+
+-- The platform dependent End Of Line string
+-- e.g. this should be changed to "\n" under UNIX, etc.
+local EOL = "\r\n"
+
+-- The raw logging function
+--
+log = log or {} -- fast hack to make the xml generator happy
+log.raw = function ( line, data )
+  if not ENABLE_LOGRAW then
+    return
+  end
+
+  local out = assert(io.open("log_raw.txt", "ab"))
+  out:write( EOL .. os.date("%c") .. " : " )
+  out:write( line )
+  if data ~= nil then
+    out:write( EOL .. "--------------------------------------------------" .. EOL )
+    out:write( data )
+    out:write( EOL .. "--------------------------------------------------" )
+  end
+  assert(out:close())
+end
+
+
 -- ************************************************************************** --
 --  Helper functions
 -- ************************************************************************** --
@@ -193,12 +241,35 @@ end
 function hash()
   return (internalState.strUser or "") .. "~" ..
          (internalState.strDomain or "") .. "~"  ..
-         (internalState.strMBox or "")
+         (string.sub(internalState.strMBox, -5) or "")
+end
+
+function getPage(browser, url)
+  local try = 0
+  while (try < 3) do
+    local body, err = browser:get_uri(url)
+
+    if (body == nil) then
+      log.raw("Tried to load: " .. url .. " and got error: " .. err)
+    elseif (string.find(body, "We are experiencing higher than normal volume") == nil and 
+        string.find(body, "<[Hh][Tt][Mm][Ll]>") ~= nil and
+        string.find(body, "MSN Hotmail %- ERROR") == nil) then
+      return body, err
+    end
+    try = try + 1
+    if (body ~= nil) then
+      log.raw("Attempt to load: " .. url .. " failed in attempt: " .. try)
+    end
+  end
+
+  return nil, nil
 end
 
 -- Issue the command to login to Hotmail
 --
 function loginHotmail()
+  log.raw("Entering login")
+
   -- Check to see if we've already logged in
   --
   if internalState.loginDone then
@@ -216,7 +287,6 @@ function loginHotmail()
   local passwordlen = string.len(internalState.strPassword)
   local domain = internalState.strDomain
   local url = globals.strLoginUrl
-  local xml = globals.strFolderQry
   local browser = internalState.browser
 	
   -- DEBUG - Set the browser in verbose mode
@@ -229,12 +299,13 @@ function loginHotmail()
 
   -- Retrieve the login page.
   --
-  local body, err = browser:get_uri(url)
+  local body, err = getPage(browser, url)
 
   -- No connection
   --
   if body == nil then
     log.error_print("Login Failed: Unable to make connection")
+    log.raw("Login failed: Can't get the page: " .. url .. ", Error: " .. (err or "none"));
     return POPSERVER_ERR_NETWORK
   end
 
@@ -264,38 +335,48 @@ function loginHotmail()
   local _, _, str2 = string.find(body, globals.strLoginPostUrlPattern5)
   if (url == nil or str == nil or str2 == nil) then
     log.error_print(globals.strLoginFailed)
-    return POPSERVER_ERR_AUTH
+    return POPSERVER_ERR_NETWORK
   end
   url = url .. "?" .. str 
 
   local padding = string.sub(globals.strLoginPaddingFull, 0, 16 - passwordlen)
-  postdata = string.format(globals.strLoginPostData, username, domain, password, padding)
+  postdata = string.format(globals.strLoginPostData, username, domain, curl.escape(password), padding)
   postdata = postdata .. "&PPFT=" .. str2
 
   log.dbg("Hotmail - Sending login information to: " .. url)
   body, err = browser:post_uri(url, postdata)
+  if (body == nil) then
+      log.error_print(globals.strLoginFailed)
+      log.raw("Login failed: Sent login info to: " .. (url or "none") .. " and got something we weren't expecting(0):\n" .. err);
+      return POPSERVER_ERR_NETWORK
+  end
 
   -- We should be logged in now!  Unfortunately, we aren't done.  Hotmail returns a page
   -- that should auto-reload in a browser but not in curl.  It's the URL for Hotmail Today.
   --
-  _, _, url = string.find(body, globals.strLoginDoneReloadToHMHome)
+  _, _, url = string.find(body, globals.strLoginDoneReloadToHMHome1)
   if url == nil then
-    log.error_print(globals.strLoginFailed)
-    return POPSERVER_ERR_AUTH
+    _, _, url = string.find(body, globals.strLoginDoneReloadToHMHome2)
+    if url == nil then
+      log.error_print(globals.strLoginFailed)
+      log.raw("Login failed: Sent login info to: " .. (url or "none") .. " and got something we weren't expecting(1):\n" .. body);
+      return POPSERVER_ERR_NETWORK
+    end
   end
 
   _, _, str = string.find(url, globals.strRetLoginBadLogin)
   if str ~= nil then
     log.error_print(globals.strLoginFailed)
-    return POPSERVER_ERR_AUTH
+    log.raw("Login failed: Sent login info to: " .. (url or "none") .. " and got something we weren't expecting(2):\n" .. body);
+    return POPSERVER_ERR_NETWORK
   end
-  body, err = browser:get_uri(url)
+  body, err = getPage(browser, url)
 
   -- Shouldn't happen but you never know
   --
   if body == nil then
     log.error_print(globals.strLoginFailed)
-    return POPSERVER_ERR_AUTH
+    return POPSERVER_ERR_NETWORK
   end
 
   -- Save the mail server
@@ -329,6 +410,7 @@ function loginHotmail()
   else
     internalState.strImgServer = internalState.strMailServer
     log.dbg("Couldn't figure out the image server.  Using the mail server as a default.")
+    log.raw("Login failed: Should be logged in right now but am not.  Last page loaded: " .. url .. " with body:\n" .. body);
   end
 
   -- See if we are using the new folder id's
@@ -344,6 +426,22 @@ function loginHotmail()
   --
   internalState.loginTime = os.clock();
 
+  -- If we haven't set the folder yet, then it is a custom one and we need to grab it
+  --
+  if (internalState.strMBoxName ~= nil) then
+    local url = string.format(globals.strCmdFolders, internalState.strMailServer, 
+      internalState.strCrumb)
+    body, err = browser:get_uri(url)
+    _, _, str = string.find(body, globals.strFolderPattern .. internalState.strMBoxName .. "</a>")
+    if (str == nil) then
+      log.error_print(globals.strLoginFailed)
+      return POPSERVER_ERR_NETWORK
+    else
+      internalState.strMBox = str
+      log.dbg("Hotmail - Using custom folder (" .. internalState.strMBox .. ")")
+    end
+  end
+
   -- Note that we have logged in successfully
   --
   internalState.bLoginDone = true
@@ -355,6 +453,7 @@ function loginHotmail()
 
   -- Return Success
   --
+  log.raw("Successful login")
   return POPSERVER_ERR_OK
 end
 
@@ -388,6 +487,8 @@ function downloadMsg(pstate, msg, nLines, data)
     -- Whether this is the first call of the callback
     --
     bFirstBlock = true,
+    nAttempt = 0,
+    bRetry = true,
 
     -- String hacker
     --
@@ -403,7 +504,12 @@ function downloadMsg(pstate, msg, nLines, data)
 
     -- Buffer
     --
-    strBuffer = ""
+    strBuffer = "",
+
+    -- addition
+    cb_uidl = uidl,     --  to provide the uidl for later... 
+    bEndReached = false  	-- this is a hack, I know...
+    -- end of addition
   }
 	
   -- Define the callback
@@ -412,10 +518,17 @@ function downloadMsg(pstate, msg, nLines, data)
 
   -- Start the download on the body
   -- 
-  local f, _ = browser:pipe_uri(url, cb)
-  if not f then
-    -- An empty message.  Throw an error
-    --
+  while (cbInfo.bRetry == true and cbInfo.nAttempt < 3) do
+    local f, err = browser:pipe_uri(url, cb)
+    if err then
+      -- Log the error
+      --
+      log.raw("Message: " .. cbInfo.cb_uidl .. " received error: " .. err)
+    end
+  end
+  if (cbInfo.bRetry == true) then
+    log.error_print("Message: " .. cbInfo.cb_uidl .. " failed to download.")
+    log.raw("Message: " .. cbInfo.cb_uidl .. " failed to download.")
     return POPSERVER_ERR_NETWORK
   end
 
@@ -423,15 +536,20 @@ function downloadMsg(pstate, msg, nLines, data)
   --
   local body = cbInfo.strBuffer
   if (string.len(body) > 0) then
-    body = cleanupBody(body)
+    log.raw("Message: " .. cbInfo.cb_uidl .. ", left over buffer being processed: " .. body)
+    body = cleanupBody(body, cbInfo)
     body = cbInfo.strHack:dothack(body) .. "\0"
     popserver_callback(body, data)
   end
 
   -- Mark the message as read
   --
-  browser:get_head(markReadUrl)
+  if internalState.bMarkMsgAsUnread == false then
+    log.raw("Message: " .. cbInfo.cb_uidl .. ", Marking message as being done.")
+    browser:get_head(markReadUrl)
+  end
 
+  log.raw("Message: " .. cbInfo.cb_uidl .. ", Completed!")
   return POPSERVER_ERR_OK
 end
 
@@ -440,9 +558,27 @@ end
 function downloadMsg_cb(cbInfo, data)
 	
   return function(body, len)
+    log.raw("In download callback: " .. cbInfo.cb_uidl)    
+
+    -- Is this the first block?  If so, make sure we have a valid message
+    --
+    if (cbInfo.bFirstBlock == true) then
+      if (string.find(body, "<pre>")) then
+        cbInfo.bRetry = false
+        log.raw("Message: " .. cbInfo.cb_uidl .. " was loaded successfully.")
+      else
+        cbInfo.bRetry = true   
+        cbInfo.nAttempt = cbInfo.nAttempt + 1
+        log.raw("Message: " .. cbInfo.cb_uidl .. " failed on attempt: " .. cbInfo.nAttempt .. ", Body: " .. body)
+        log.raw("out download callback: " .. cbInfo.cb_uidl)    
+        return 0, nil
+      end
+    end 
+
     -- Are we done with Top and should just ignore the chunks
     --
-    if (cbInfo.nLinesRequested ~= -2 and cbInfo.nLinesReceived == -1) then
+    if (cbInfo.nLinesReceived == -1) then
+      log.raw("out download callback: " .. cbInfo.cb_uidl .. ", Sent: <Nothing>")    
       return 0, nil
     end
 
@@ -450,18 +586,17 @@ function downloadMsg_cb(cbInfo, data)
     --
     body = cbInfo.strBuffer .. body
     cbInfo.strBuffer = ""
-    while (string.len(body) > 6 and string.find(string.sub(body, -6), "([&<])") ~= nil) do
-      cbInfo.strBuffer = string.sub(body, -6) .. cbInfo.strBuffer
-      body = string.sub(body, 1, -7)
+    if (string.len(body) > 6) then
+      local idx = string.find(string.sub(body, -6), "([&<])")
+      if (idx ~= nil) then
+        idx = idx - 1
+        cbInfo.strBuffer = string.sub(body, -6 + idx)
+        local len = string.len(body) - 6 + idx
+        body = string.sub(body, 1, len)
+      end
     end
 
-    -- Did we reach the end of the message
-    --
-    if (string.find(body, "(</pre>)") ~= nil) then
-      cbInfo.nLinesReceived = -1;
-    end
-
-    body = cleanupBody(body)
+    body = cleanupBody(body, cbInfo)
 
     -- Perform our "TOP" actions
     --
@@ -489,15 +624,86 @@ function downloadMsg_cb(cbInfo, data)
     --
     popserver_callback(body, data)
 			
+    log.raw("out download callback: " .. cbInfo.cb_uidl .. ", Sent: " .. body)    
     return len, nil
   end
 end
 
-function cleanupBody(body)
+function cleanupHeaders(headers, cbInfo)
+  -- Cleanup the headers.  They are HTML-escaped.  
+  --
+  local origHeaders = headers
+  local bMissingTo = false    -- it seems hotmail server sometimes disregards To: when 'raw=0' ? 
+                              --   probably only in internal HoTMaiL-service letters, where the actual 
+                              --   internet message headers do not contain actual To:-field
+  local bMissingID = false    -- when no Message-ID -field seems to have been automatically generated ?
+  local bodyrest = ""
+
+  _, _, headers, bodyrest = string.find(headers, "^(.-)\r*\n%s*\r*\n(.*)$" )
+
+  if (headers == nil) then
+    log.dbg("Hotmail: unable to parse out message headers.  Extra headers will not be used.")
+    return origHeaders
+  end  
+
+  headers = string.gsub(headers, "%s+$", "\n")
+  headers = headers .. "\n";
+  headers = string.gsub(headers, "\n\n", "\n")
+
+  --
+  -- some checking...
+  --
+  if string.find(headers, "(To:)") == nil then
+    bMissingTo = true
+  end
+  if string.find(headers, "(Message%-I[dD]:)") == nil then
+    bMissingID = true
+  end
+
+  -- Add some headers
+  --
+  if bMissingTo ~= false then
+    headers = headers .. "To: " .. internalState.strUser .. "@" .. internalState.strDomain .. "\r\n" ;
+  end
+
+  if bMissingID ~= false then
+    local msgid = cbInfo.cb_uidl .. "@" .. internalState.strMailServer -- well, if we do not have any better choice...
+    headers = headers .. "Message-ID: <" .. msgid .. ">\r\n"  
+  end
+
+  headers = headers .. "X-FreePOPs-User: " .. internalState.strUser .. "@" .. internalState.strDomain .. "\r\n"
+  headers = headers .. "X-FreePOPs-Domain: " .. internalState.strDomain .. "\r\n"
+  headers = headers .. "X-FreePOPs-Folder: " .. internalState.strMBox .. "\r\n"
+  headers = headers .. "X-FreePOPs-MailServer: " .. internalState.strMailServer .. "\r\n"
+  headers = headers .. "X-FreePOPs-ImageServer: " .. internalState.strImgServer .. "\r\n"
+  headers = headers .. "X-FreePOPs-MsgNumber: " .. "<" .. cbInfo.cb_uidl .. ">" .. "\r\n"
+
+  -- make the final touch...
+  --
+  headers = headers .. "\r\n" .. bodyrest
+
+  return headers 
+end
+
+
+function cleanupBody(body, cbInfo)
+  -- check to see whether the end of message has already been seen...
+  --
+  if (cbInfo.bEndReached == true) then
+    return ("") ; 	-- in this case we pass nothing past the end of message
+  end
+
+  -- Did we reach the end of the message
+  --
+  if (string.find(body, "(</pre>)") ~= nil) then
+    cbInfo.nLinesReceived = -1;
+    cbInfo.bEndReached = true 
+  end
+
   -- The only parts we care about are within <pre>..</pre>
   --
   body = string.gsub(body, "<pre>[%s]*", "")
-  body = string.gsub(body, "</pre>.*", "")
+  body = string.gsub(body, "</pre>.-$", "")
 
   -- Clean up the end of line, and replace HTML tags
   --
@@ -506,7 +712,17 @@ function cleanupBody(body)
   body = string.gsub(body, "&lt;", "<")
   body = string.gsub(body, "&gt;", ">")
   body = string.gsub(body, "&quot;", "\"")
+  body = string.gsub(body, "&#34;", "\"")
+  body = string.gsub(body, "&nbsp;", " ")
   body = string.gsub(body, "<!%-%-%$%$imageserver%-%->", internalState.strImgServer)
+
+  -- We've now at least seen one block, attempt to clean up the headers
+  --
+  if (cbInfo.bFirstBlock == true) then
+    cbInfo.bFirstBlock = false 
+    body = cleanupHeaders(body, cbInfo)
+  end
+
   return body
 end
 
@@ -535,6 +751,15 @@ function user(pstate, username)
     internalState.bEmptyTrash = true
   end
 
+  -- If the flag markunread=1 is set, then we will mark all messages
+  -- that we pull as unread when done.
+  --
+  local val = (freepops.MODULE_ARGS or {}).markunread or 0
+  if val == "1" then
+    log.dbg("Hotmail: All messages pulled will be marked unread.")
+    internalState.bMarkMsgAsUnread = true
+  end
+
   -- Get the folder
   --
   local mbox = (freepops.MODULE_ARGS or {}).folder
@@ -567,13 +792,10 @@ function user(pstate, username)
     return POPSERVER_ERR_OK
   end
 
-  -- TODO - set the other mailbox here and find it
-  -- when we log in.
-  -- 
-
   -- Defaulting to the inbox
   --
-  log.say("Unable to figure out the mailbox specified.  Defaulting to the Inbox.\n")
+  log.say("Custom mailbox set to: " .. mbox .. ".\n")
+  internalState.strMBoxName = mbox
   internalState.strMBox = globals.strInbox
   return POPSERVER_ERR_OK
 end
@@ -696,7 +918,7 @@ function quit_update(pstate)
     if internalState.strCrumb ~= '' then
       cmdUrl = string.format(globals.strCmdEmptyTrash, internalState.strMailServer,internalState.strCrumb)
       log.dbg("Sending Empty Trash URL: " .. cmdUrl .."\n")
-      local body, err = browser:get_uri(cmdUrl)
+      local body, err = getPage(browser, cmdUrl)
       if not body or err then
         log.error_print("Error when trying to empty the trash with url: ".. cmdUrl .."\n")
       end
@@ -713,11 +935,13 @@ function quit_update(pstate)
   if diff > globals.nSessionTimeout then 
     cmdUrl = string.format(globals.strCmdLogout, internalState.strMailServer)
     log.dbg("Sending Logout URL: " .. cmdUrl .. "\n")
-    local body, err = browser:get_uri(cmdUrl)
+    local body, err = getPage(browser, cmdUrl)
  
     log.dbg("Logout forced to keep hotmail session fresh and tasty!  Yum!\n")
     log.dbg("Session removed - Account: " .. internalState.strUser .. 
       "@" .. internalState.strDomain .. "\n")
+    log.raw("Session removed (Forced by Hotmail timer) - Account: " .. internalState.strUser .. 
+      "@" .. internalState.strDomain) 
     session.remove(hash())
     return POPSERVER_ERR_OK
   end
@@ -873,7 +1097,7 @@ function stat(pstate)
 
     -- Get the page and check to see if we got results
     --
-    local body, err = browser:get_uri(cmdUrl)
+    local body, err = getPage(browser, cmdUrl)
     if body == nil then
       return body, err
     end
@@ -893,6 +1117,7 @@ function stat(pstate)
       --
       internalState.bLoginDone = nil
       session.remove(hash())
+      log.raw("Session Expired - Last page loaded: " .. cmdUrl .. ", Body: " .. body)
 
       -- Try Logging back in
       --
@@ -912,7 +1137,7 @@ function stat(pstate)
 
       -- Retry to load the page
       --
-      return browser:get_uri(cmdUrl)
+      return getPage(browser, cmdUrl)
     end
 
     -- Get the total number of messages
@@ -949,6 +1174,8 @@ function stat(pstate)
   if not support.do_until(funcGetPage, funcCheckForMorePages, funcProcess) then
     log.error_print("STAT Failed.\n")
     session.remove(hash())
+    log.raw("Session removed (STAT Failure) - Account: " .. internalState.strUser .. 
+      "@" .. internalState.strDomain) 
     return POPSERVER_ERR_NETWORK
   end
 	
