@@ -17,7 +17,7 @@
 -- ************************************************************************** --
 --  FreePOPs @libero.it, @inwind.it, @blu.it, @iol.it webmail interface
 --  
---  $Id: libero.lua,v 1.39 2005/06/27 17:58:09 gareuselesinge Exp $
+--  $Id: libero.lua,v 1.46 2005/10/24 22:26:39 alessiofender Exp $
 --  
 --  Released under the GNU/GPL license
 --  Written by Enrico Tassi <gareuselesinge@users.sourceforge.net>
@@ -27,7 +27,7 @@
 -- fill them in the right way
 
 -- single string, all required
-PLUGIN_VERSION = "0.2.1"
+PLUGIN_VERSION = "0.2.5"
 PLUGIN_NAME = "Libero.IT"
 PLUGIN_REQUIRE_VERSION = "0.0.14"
 PLUGIN_LICENSE = "GNU/GPL"
@@ -98,17 +98,16 @@ local libero_string = {
 	-- to cut and paste the html source and work on it. For example
 	-- you could copy a message table row in a blank file, substitute
 	-- every useless field with '.*'.
-	statE = ".*<script>TREvidenceMsg</script>.*<TD>.*<input.*value.*=.*[[:digit:]]+.*>.*</TD>.*<TD>.*<IMG>.*</TD>.*<TD>.*<IMG>.*</TD>.*<TD>.*<IMG>.*</TD>.*<TD>.*<img>.*</TD>.*<TD>.*<IMG>.*</TD>.*<TD>.*<a>.*<script>IMGEvidenceMsg</script>.*</a>.*<script>AEvidenceMsg</script>[.*]{b}.*{/b}[.*]</a>.*</TD>.*<TD>.*<IMG>.*</TD>.*<TD>.*<script>AEvidenceMsg</script>[.*]{b}.*{/b}[.*]</a>.*</TD>.*<TD>.*<IMG>.*</TD>.*<script>TDEvidenceMsg</script>.*</TD>.*<TD>.*<IMG>.*</TD>.*<script>TDEvidenceMsg</script>[.*]{b}.*[[:digit:]]+.*{/b}[.*]</TD>.*</TR>",
+	statE = ".*<script>.*TREv.*</script>.*<TD.*>.*<input.*value.*=.*[[:digit:]]+.*>.*</TD>.*<TD>.*<div.*>.*</TD>.*<TD.*>.*<IMG.*>.*</TD>.*<TD>.*<div.*>.*</TD>.*<TD.*>.*<[Iid][Mmi][Ggv].*>.*</TD>.*<TD>.*<div.*>.*</TD>.*<TD.*>.*<a>.*<script>IMGEv.*</script>.*</a>.*<script>AEv.*</script>[.*]{b}.*{/b}[.*]</a>.*</TD>.*<TD>.*<div.*>.*</TD>.*<TD.*>.*<script>.*AEv.*</script>[.*]{b}.*{/b}[.*]</a>.*</TD>.*<TD>.*<div.*>.*</TD>.*<script>.*TDEv.*</script>[.*]{b}.*{/b}[.*]</TD>[.*]{!--.*--}.*<TD.*>.*<div.*>.*</TD>.*<script>TDEv.*</script>[.*]{b}.*[[:digit:]]+.*{/b}[.*]</TD>[.*]{!--.*--}.*</TR>",
 	-- This is the mlex get expression to choose the important fields 
 	-- of the message list page. Used in combination with statE
-	statG = "O<O><O>O<O>O<X>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O><O>O<O>O<O><O>[O]{O}O{O}[O]<O>O<O>O<O>O<O>O<O>O<O>O<O><O>[O]{O}O{O}[O]<O>O<O>O<O>O<O>O<O>O<O><O>O<O>O<O>O<O>O<O>O<O><O>[O]{O}X{O}[O]<O>O<O>",
+	statG = "O<O><O>O<O>O<X>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O>O<O><O>O<O>O<O><O>[O]{O}O{O}[O]<O>O<O>O<O>O<O>O<O>O<O>O<O><O>[O]{O}O{O}[O]<O>O<O>O<O>O<O>O<O>O<O><O>[O]{O}O{O}[O]<O>[O]{O}O<O>O<O>O<O>O<O><O>[O]{O}X{O}[O]<O>[O]{O}O<O>",
 	-- The uri for the first page with the list of messages
 	first = "http://%s/cgi-bin/webmail.cgi?ID=%s&Act_Msgs=1&"..
 		"C_Folder=%s",
 	-- The capture to check if there is one more page of message list
-
-	next_checkC = "<a%s+href=\"javascript:doit"..
-		"%('Act_Msgs_Page_Next',1,1%)\".*</a>",
+	next_checkC = "<a href=\"javascript:doit"..
+		"%('Act_Msgs_Page_Next',1,1%)\"",
 	-- The uri to get the next page of messages
 	next = "http://%s/cgi-bin/webmail.cgi?ID=%s&Act_Msgs_Page_Next=1&"..
 		"HELP_ID=inbox&SEL_ALL=0&"..
@@ -216,7 +215,10 @@ function libero_login()
 
 	-- build the uri
 	local password = internal_state.password
-	local popnumber = math.mod(os.time(),16) + 1
+	local popnumber = math.mod(os.time(),29) + 1  -- == random(1..29)
+        if popnumber == 25 then
+        	popnumber = 24	-- wpop25 doesn't exists
+        end
 	local domain = internal_state.domain
 	local site = libero_domain[domain].website
 	local choice = libero_domain[domain].choice
@@ -226,6 +228,7 @@ function libero_login()
 	end
 	local x,y = math.mod(os.time(),16),math.mod(os.time()*2,16)
 	local uri = string.format(libero_string.login_url,popnumber,site)
+        log.dbg("Using webserver " .. uri);
 	local post= string.format(libero_string.login_post,
 	            domain,user,password,choice,x,y)
 	
