@@ -7,11 +7,11 @@
 
 -- Globals
 --
-PLUGIN_VERSION = "0.0.9a"
+PLUGIN_VERSION = "0.0.9d"
 PLUGIN_NAME = "mail.com"
 PLUGIN_REQUIRE_VERSION = "0.0.17"
 PLUGIN_LICENSE = "GNU/GPL"
-PLUGIN_URL = "http://www.freepops.org/download.php?contrib=mailcom.lua"
+PLUGIN_URL = "http://www.freepops.org/download.php?file=mailcom.lua"
 PLUGIN_HOMEPAGE = "http://www.freepops.org/"
 PLUGIN_AUTHORS_NAMES = {"Russell Schwager"}
 PLUGIN_AUTHORS_CONTACTS = 
@@ -70,7 +70,7 @@ PLUGIN_DOMAINS = {"@mail.com","@email.com","@iname.com","@cheerful.com","@consul
 "@webcity.ca","@webmail.lu","@welcomm.ac","@wenxuecity.net","@westham-mail.com","@wimbledon-mail.com",
 "@windrivers.net","@wolves-mail.com","@wongfaye.com","@worldmail.ac","@worldweb.ac","@isleuthmail.com",
 "@x-lab.cc","@xy.com.tw","@yankeeman.com","@yyhmail.com", "@verizonmail.com", "@lycos.com", "@cyberdude.com",
-"@mail.org" }
+"@mail.org", "@italymail.com" }
 PLUGIN_PARAMETERS = {
 	{name = "folder", description = {
 		en = [[
@@ -113,6 +113,7 @@ local globals = {
   strLoginPage = "http://%s.%s/scripts/common/proxy.main",
   strLoginPage2 = "http://%s.%s/scripts/common/login.main",
   strLoginPage3 = "http://%s.%s/scripts/lycos/lyproxy.main",
+  strLoginPage4 = "http://super.japan.com/scripts/common/ss_main.cgi",
   strLoginPostData = "show_frame=Enter&mail_language=us&action=login&login=%s%%40%s&password=%s&domain=%s&siteselected=normal&",
   strLoginPostData2 = "show_frame=Enter&mail_language=us&action=login&login=%s&password=%s&domain=%s&siteselected=normal&",
 
@@ -120,8 +121,8 @@ local globals = {
 
   -- Expressions to pull out of returned HTML from mail.com corresponding to a problem
   --
-  strRetLoginBadPassword = "(Invalid username/password.)",
-  strRetLoginSessionExpired = "( [Mm]essage%(s%))",
+  strRetLoginBadPassword = "(Invalid username[^p]+password.)",
+  strRetLoginSessionExpired = "( [Mm]essage)",
 
   -- Regular expression to extract the mail server
   --
@@ -138,6 +139,7 @@ local globals = {
   --
 --  strMsgListCntPattern = "Showing [^%d]*[%d]+[^%s]* to [^%d]*[%d]+[^%s]* of [^%d]*([%d]+)",
   strMsgListCntPattern = "(%d+) [Mm]essage%(s%)",
+  strMsgListCntPattern2 = "(%d+) Total Messages",
 
   -- Defined Mailbox names - These define the names to use in the URL for the mailboxes
   --
@@ -172,6 +174,37 @@ internalState = {
   bOptionOverride = false,
 }
 
+-- ************************************************************************** --
+--  Logging functions
+-- ************************************************************************** --
+
+-- Set to true to enable Raw Logging
+--
+local ENABLE_LOGRAW = false
+
+-- The platform dependent End Of Line string
+-- e.g. this should be changed to "\n" under UNIX, etc.
+local EOL = "\r\n"
+
+-- The raw logging function
+--
+log = log or {} -- fast hack to make the xml generator happy
+log.raw = function ( line, data )
+  if not ENABLE_LOGRAW then
+    return
+  end
+
+  local out = assert(io.open("log_raw.txt", "ab"))
+  out:write( EOL .. os.date("%c") .. " : " )
+  out:write( line )
+  if data ~= nil then
+    out:write( EOL .. "--------------------------------------------------" .. EOL )
+    out:write( data )
+    out:write( EOL .. "--------------------------------------------------" )
+  end
+  assert(out:close())
+end
+
 -- ************************************************************************** --
 --  Helper functions
 -- ************************************************************************** --
@@ -233,7 +266,6 @@ function postToLoginPage(browser, url, post, attempt)
     --
     log.dbg("Mail.com Mail Server: " .. str .. "\n")
   end
-
   return POPSERVER_ERR_OK
 end
 
@@ -268,6 +300,8 @@ function login()
   elseif (domain == "otakumail.com") then
     post = string.format(globals.strLoginPostData2, username, password, domain)
     url = string.format(globals.strLoginPage2, "www", "otakumail.com")
+  elseif (domain == "japan.com") then
+    url = globals.strLoginPage4
   else
     url = string.format(globals.strLoginPage, "www", domain)
   end
@@ -756,13 +790,17 @@ function stat(pstate)
 
       -- Retry to load the page
       --
-      return getPage(browser, cmdUrl)
+      body, err = getPage(browser, cmdUrl)
     end
 
     -- Get the total number of messages
     --
     if nTotMsgs == 0 then
       _, _, nTotMsgs = string.find(body, globals.strMsgListCntPattern)
+      if nTotMsgs == nil then -- Try a different pattern
+        _, _, nTotMsgs = string.find(body, globals.strMsgListCntPattern2)
+      end
+
       if nTotMsgs == nil then
         nTotMsgs = 0
       else 
