@@ -8,7 +8,7 @@
 
 -- Globals
 --
-PLUGIN_VERSION = "0.1.6"
+PLUGIN_VERSION = "0.1.7b"
 PLUGIN_NAME = "yahoo.com"
 PLUGIN_REQUIRE_VERSION = "0.0.27"
 PLUGIN_LICENSE = "GNU/GPL"
@@ -18,7 +18,11 @@ PLUGIN_AUTHORS_NAMES = {"Russell Schwager","Nicola Cocchiaro"}
 PLUGIN_AUTHORS_CONTACTS = 
 	{"russells (at) despammed (.) com",
          "ncocchiaro (at) users (.) sourceforge (.) net"}
-PLUGIN_DOMAINS = {"@yahoo.com","@yahoo.it", "@yahoo.ca", "@rocketmail.com"}
+PLUGIN_DOMAINS = {"@yahoo.com","@yahoo.it", "@yahoo.ca", "@rocketmail.com", "@yahoo.com.ar",
+                  "@yahoo.co.in", "@yahoo.com.tw", "@yahoo.co.uk", "@yahoo.com.cn",
+                  "@yahoo.es", "@yahoo.de", "@talk21.com", "@btinternet.com"
+}
+
 PLUGIN_PARAMETERS = {
 	{name = "folder", description = {
 		it = [[
@@ -92,8 +96,8 @@ name and your real password as the password.]]
 
 local globals = {
   -- Login strings
-  -- TODO: Define the HTTPS version
   --
+  strMailPage = "http://mail.yahoo.com",
   strLoginPage = "http://login.yahoo.com",
   strLoginHTTP = "http://login.yahoo.com/config/login",   
   strLoginHTTPs = "https://login.yahoo.com/config/login",   
@@ -113,7 +117,6 @@ local globals = {
 
   -- Get the mail server for Yahoo
   --
-  --strRegExpMailServer = '<a href="(http://[^"]*)ym/',
   strRegExpMailServer = '(http://[^/]+/)ym/',
   
   -- Redirect site on login
@@ -126,8 +129,16 @@ local globals = {
 
   -- Get the crumb value that is needed for deleting messages and emptying the trash
   --
-  strRegExpCrumb = "=1&%.crumb=([^&]*)&",
-  strRegExpCrumb2 = "&UNR=1&.crumb=([^&]*)&",
+  strRegExpCrumb = '<input type=hidden name=".crumb" value="([^"]+)">',
+
+  -- Delete Form hidden items
+  --
+  strHiddenItems = '<input type="hidden" name="([^"]+)".- value="([^"]-)">',
+  strDeletePostPat = '<form name=messageList method=post action="/([^"]+)">',
+
+  -- Mark unread url
+  -- 
+  strMsgMarkUnreadPat = '<a href="/(ym[^"]*UNR=1[^"]*)">',
 
   -- Pattern to determine if we have no messages.  If this is found, we have messages.
   --
@@ -146,6 +157,10 @@ local globals = {
   --
   strMsgListPrevPagePattern = '<a href="/(ym[^"]*previous=1[^"]*)">',
 
+  -- Pattern for emptying all
+  --
+  strEmptyAllPat = 'url %+= "=1&([^"]+)"',
+
   -- Defined Mailbox names - These define the names to use in the URL for the mailboxes
   --
   strInbox = "Inbox",
@@ -167,9 +182,8 @@ local globals = {
   strCmdMsgList = "%sym/ShowFolder?box=%s&Npos=%d&Nview=%s&order=up&sort=date&reset=1&Norder=up",
   strCmdMsgView = "%sym/ShowLetter?box=%s&PRINT=1&Nhead=f&toc=1&MsgId=%s&bodyPart=%s",
   strCmdMsgWebView = "%sym/ShowLetter?box=%s&MsgId=%s",
-  strCmdDelete = "%sym/ShowFolder?box=%s&DEL=Delete", -- &Mid=%s&.crumb=%s
-  strCmdEmptyTrash = "%sym/ShowFolder?ET=1&.crumb=%s&reset=1", 
-  strCmdEmptyBulk = "%sym/ShowFolder?EB=1&.crumb=%s&reset=1", 
+  strCmdEmptyTrash = "%sym/ShowFolder?ET=1&", 
+  strCmdEmptyBulk = "%sym/ShowFolder?EB=1&", 
   strCmdUnread = "%sym/ShowLetter?box=%s&MsgId=%s&.crumb=%s&UNR=1",
 
   -- Emails to list - These define the filter on the messages to grab
@@ -222,7 +236,6 @@ internalState = {
   browser = nil,
   strMailServer = nil,
   strDomain = nil,
-  strCrumb = nil,
   strMBox = nil,
   strIntFlag = nil,
   strView = nil,
@@ -230,7 +243,8 @@ internalState = {
   bNoSSL = false,
   bEmptyTrash = false,
   bEmptyBulk = false,
-  msgids = {}
+  msgids = {},
+  strStatCache = nil
 }
 
 -- ************************************************************************** --
@@ -258,6 +272,7 @@ local EOL = "\r\n"
 
 -- The raw logging function
 --
+log = log or {} -- fast hack to make the xml generator happy
 log.raw = function ( line, data )
   if not ENABLE_LOGRAW then
     return
@@ -368,6 +383,8 @@ function loginYahoo()
   if (domain == "rocketmail.com") then
     domain = "yahoo.com"
     username = username .. ".rm"
+  elseif (domain == "btinternet.com" or domain == "talk21.com") then
+    username = username .. "@" .. domain
   end
 	
   -- DEBUG - Set the browser in verbose mode
@@ -417,6 +434,9 @@ function loginYahoo()
   -- 
   local _, _, str = string.find(body, globals.strRetLoginBadPassword)
   if str ~= nil then
+    log.raw("------ Returned Page saying invalid password ------\n")
+    log.raw(body)
+    log.raw("------ End Page saying invalid password -------\n")
     log.error_print("Login Failed: Invalid Password")
     return POPSERVER_ERR_AUTH
   end
@@ -550,12 +570,12 @@ function downloadYahooMsg(pstate, msg, nLines, data)
     local cmdUrl = string.format(globals.strCmdMsgWebView, internalState.strMailServer,
       internalState.strMBox, msgid);
     local str, _ = browser:get_uri(cmdUrl) 
-    _, _, str = string.find(str, globals.strRegExpCrumb2)
+    _, _, str = string.find(str, globals.strMsgMarkUnreadPat)
     if str == nil then
-      log.warn("Unable to get the crumb view for marking message as unread.")
+      log.warn("Unable to get the url for marking message as unread.")
     else
-      cmdUrl = string.format(globals.strCmdUnread, internalState.strMailServer,
-        internalState.strMBox, msgid, str);
+      cmdUrl = internalState.strMailServer .. str;
+      log.dbg("Marking as message: " .. msgid .. " as unread, url: " .. cmdUrl);
       browser:get_uri(cmdUrl) -- We don't care about the results.
     end
   end
@@ -826,9 +846,8 @@ function quit_update(pstate)
   -- Local Variables
   --
   local browser = internalState.browser
-  local baseUrl = string.format(globals.strCmdDelete, internalState.strMailServer,
-    internalState.strMBox);
-  local cmdUrl = baseUrl
+  local cmdUrl = nil
+  local postdata = ""
   local cnt = get_popstate_nummesg(pstate)
   local dcnt = 0
 
@@ -838,43 +857,63 @@ function quit_update(pstate)
     if get_mailmessage_flag(pstate, i, MAILMESSAGE_DELETE) then
       local uidl = get_mailmessage_uidl(pstate, i)
       local msgid = internalState.msgids[uidl]
-      cmdUrl = cmdUrl .. "&Mid=" .. msgid
+      postdata = postdata .. "Mid=" .. msgid .. "&"
       dcnt = dcnt + 1
-
-      -- Send out in a batch of 5
-      --
-      if math.mod(dcnt, 5) == 0 then
-        cmdUrl = cmdUrl .. "&.crumb=" .. internalState.strCrumb
-        log.dbg("Sending Delete URL: " .. cmdUrl .. "\n")
-        local body, err = browser:get_uri(cmdUrl)
-        if not body or err then
-          log.error_print("Unable to delete messages.\n")
-        end
-       
-        -- Reset the variables
-        --
-        dcnt = 0
-        cmdUrl = baseUrl
-      end
     end
   end
 
-  -- Send whatever is left over
+  -- We don't have a stat cache
   --
-  if dcnt > 0 and dcnt < 5 then
-    cmdUrl = cmdUrl .. "&.crumb=" .. internalState.strCrumb
-    log.dbg("Sending Delete URL: " .. cmdUrl .. "\n")
-    local body, err = browser:get_uri(cmdUrl)
-    if not body or err then
-      log.error_print("Unable to delete messages.\n")
+  if (internalState.strStatCache == nil) then
+    log.dbg("Yahoo - unable to retrieve the crumb value.")
+    return POPSERVER_ERR_OK
+  end
+
+  -- Let's get the crumb value
+  --
+  local _, _, strCrumb = string.find(internalState.strStatCache, globals.strRegExpCrumb)
+  if strCrumb == nil then
+    log.error_print("Yahoo - unable to parse out crumb value.  Deletion will fail.")
+    log.raw("Yahoo - unable to parse out crumb value.  Deletion will fail, Body: " .. 
+      internalState.strStatCache)
+    return POPSERVER_ERR_OK
+  end
+
+  -- We have things to delete, let's do it!
+  --
+  if (dcnt > 0) then
+    -- Lets get the form variables
+    --
+    local name, value  
+    for name, value in string.gfind(internalState.strStatCache, globals.strHiddenItems) do
+      postdata = postdata .. name .. "=" .. curl.escape(value) .. "&" 
     end
+    postdata = string.gsub(postdata, "DEL=&", "DEL=1&")
+    postdata = postdata .. ".crumb=" .. strCrumb
+   
+    -- Get the url to post to
+    --
+    _, _, cmdUrl = string.find(internalState.strStatCache, globals.strDeletePostPat)
+    if (cmdUrl == nil) then 
+      log.error_print("Yahoo - unable to parse out delete url.  Deletion will fail.")
+      log.raw("Yahoo - unable to parse out delete url.  Deletion will fail, Body: " .. 
+        internalState.strStatCache)
+      return POPSERVER_ERR_OK
+    end
+
+    -- Do it!
+    -- 
+    cmdUrl = internalState.strMailServer .. cmdUrl
+    log.dbg("Yahoo - Sending delete url: " .. cmdUrl .. ", data: " .. postdata)
+    browser:post_uri(cmdUrl, postdata)
   end
 
   -- Empty the trash
   --
+  local _, _, strAll = string.find(internalState.strStatCache, globals.strEmptyAllPat)
   if internalState.bEmptyTrash then
-    if internalState.strCrumb ~= '' then
-      cmdUrl = string.format(globals.strCmdEmptyTrash, internalState.strMailServer,internalState.strCrumb)
+    if strAll ~= nil then
+      cmdUrl = string.format(globals.strCmdEmptyTrash, internalState.strMailServer) .. strAll
       log.dbg("Sending Empty Trash URL: ".. cmdUrl .."\n")
       local body, err = browser:get_uri(cmdUrl)
       if not body or err then
@@ -888,8 +927,8 @@ function quit_update(pstate)
   -- Empty the bulk folder
   --
   if internalState.bEmptyBulk then
-    if internalState.strCrumb ~= '' then
-      cmdUrl = string.format(globals.strCmdEmptyBulk, internalState.strMailServer,internalState.strCrumb)
+    if strAll ~= nil then
+      cmdUrl = string.format(globals.strCmdEmptyBulk, internalState.strMailServer) .. strAll
       log.dbg("Sending Empty Bulk URL: ".. cmdUrl .."\n")
       local body, err = browser:get_uri(cmdUrl)
       if not body or err then
@@ -900,8 +939,6 @@ function quit_update(pstate)
     end
   end
 
-  -- Save and then Free up the session
-  --
   session.save(hash(), serialize_state(), session.OVERWRITE)
   session.unlock(hash())
 
@@ -936,6 +973,10 @@ function stat(pstate)
   --
   local knownIDs = {}
 
+  -- Force Yahoo to update
+  --
+  local body, err = browser:get_uri(globals.strMailPage)
+
   -- Debug Message
   --
   log.dbg("Stat URL: " .. cmdUrl .. "\n");
@@ -1081,29 +1122,14 @@ function stat(pstate)
       --
       browser = internalState.browser
       cmdUrl = string.format(globals.strCmdMsgList, internalState.strMailServer,
-        internalState.strMBox, nPage, globals.nViewAll);
+        internalState.strMBox, nPage, internalState.strView);
 
       -- Retry to load the page
       --
-      return browser:get_uri(cmdUrl)
-    end
-
-    -- Extract the crumb - This is needed for deletion of items
-    --
-    if (internalState.strCrumb == nil) then
-      local _, _, str = string.find(body, globals.strRegExpCrumb)
-      if str == nil then
-        log.error_print("Can't get the 'crumb' value. Will not be able to delete messages.")
-        internalState.strCrumb = ""
-      else
-        internalState.strCrumb = str
-  
-        -- Debug Message
-        -- 
-        log.dbg("Yahoo Crumb value: " .. str .. "\n")
-      end
+      browser:get_uri(cmdUrl)
     end
 		
+    internalState.strStatCache = body
     return body, err
   end
 
